#ifndef __WEATHER_STATION_PACKET_H__
#define __WEATHER_STATION_PACKET_H__

#include "crc.h"

class WeatherStationPacket
{
public:
	uint8_t getId() { return _plainBuffer[0]; } 
	void setId(uint8_t id){ _plainBuffer[0] = id;}
	
	uint8_t getType() { return _plainBuffer[1]; } 
	void setType(uint8_t type){ _plainBuffer[1] = type;}
	
	uint8_t getStatus() { return _plainBuffer[2]; } 
	void setStatus(uint8_t status){ _plainBuffer[2] = status;}
	
	uint8_t getCRC() { return _plainBuffer[3]; } 
	void setCRC(uint8_t crc){ _plainBuffer[3] = crc;}
	
	uint8_t* getPlainBuffer() { return _plainBuffer;}
	
	uint8_t* getCipherBuffer() { return _cipherBuffer;}
	
	
	WeatherStationPacket();
	
	WeatherStationPacket(uint8_t id){
		setId(id);
	}
		
	void write(uint8_t pos, const void* data, uint8_t size){
		memcpy(_plainBuffer+pos,  data, size);
	}
	
	void read(uint8_t pos, void* data, uint8_t size){
		memcpy(data, _plainBuffer+pos, size);
	}
	
	uint8_t calcCRC(){
	  uint8_t crc = 0;
	  for (uint16_t idx = 4; idx <= 31; ++idx){
		crc = crcTable[(crc ^ _plainBuffer[idx])];
	  }
	  return crc;
	}
	
	void updateCRC(){
	  setCRC(calcCRC());
	}
	
	bool isCrcValid(){
		return ( getCRC() == calcCRC() );
	}
	
	void printPlainBuffer(){
		for (int i = 0; i < 32; i++) {   
			Serial.print("0x");
			if (_plainBuffer[i] < 0x10){
				Serial.print("0");
			}
			Serial.print(_plainBuffer[i], HEX);
			Serial.print(" ");
		}   
		Serial.print("\n");
	}

	void printCipherBuffer(){
		for (int i = 0; i < 32; i++) {   
			Serial.print("0x");
			if (_cipherBuffer[i] < 0x10){
				Serial.print("0");
			}
			Serial.print(_cipherBuffer[i], HEX);
			Serial.print(" ");
		}   
		Serial.print("\n");
	}



private:
    uint8_t _plainBuffer[32];
	uint8_t _cipherBuffer[32];
};
#endif